# This script showcases the usage of the TRONTEQ ROQSTAR openAPI.
# It is compatible with ROQSTAR openAPI v1.0.
# Version of this script: 1.0.
# 
# Contact:
# www.tronteq.com
# support@tronteq.com



import binascii # base64 encoding/decoding
import time
import requests # http send/receive
import os # file access

ip = "192.168.1.1"
url_prefix = "oapi/v1"
session = requests.Session()

def _check_200(request):
    if request.status_code != 200:
        print("Error: request %s returned status code %s instead of 200" % (request.url, request.status_code))
        exit()

def _encode_base64(utf8_or_bytes):
    if type(utf8_or_bytes) is str:
        #b = bytes(utf8_or_bytes, 'utf-8')
        b = utf8_or_bytes.encode('utf-8')
    else:
        b = utf8_or_bytes
    return binascii.b2a_base64(b).decode().strip('\n')

def _load_file(file_path):
    if not os.path.exists(file_path):
        print("file '%s' does not exists" % file_path)
        fdata = None
    with open(file_path, "r") as fh:
        try:
            fdata = fh.read()
        except Exception as e:
            print("file '%s' load failed '%s'" % (file_path, str(e)))
            fdata = None
    return fdata

def get_oapi_version():
    response = requests.get("http://%s/%s/openapi/version" % (ip, url_prefix))
    _check_200(response)
    print(response.text)

def login():
    creds = {'user':'admin', 'password':'password'}
    response = session.post("http://%s/%s/login" % (ip, url_prefix), json=creds)
    _check_200(response)

def get_system_id():
    response = session.get("http://%s/%s/system/id" % (ip, url_prefix))
    _check_200(response)
    print(response.text)
    return response.json()

def print_running_config():
    response = session.get("http://%s/%s/configuration/running" % (ip, url_prefix))
    _check_200(response)
    print(response.text)
    return response.json()

def _poll_configuration_status(id):
    # Poll the status until it is 'ready'.
    while True:
        try:
            response = requests.get("http://%s/%s/openapi/status/configuration" % (ip, url_prefix))
            if response.status_code == 200:
                res = response.json()
                # Does status refer to this request (identified by '#id')?
                if res['id'] > id:
                    print("status is unclear, some other action is running already")
                    break
                else:
                    if res['status'] == 'ready':
                        break
        except Exception as e:
            pass
        time.sleep(1)

def upload_file_to_internal_storage(file_path):
    # get file name and content
    dir_path, file_name = os.path.split(file_path)
    content = _load_file(file_path)
    b64 = _encode_base64(content)

    # send request
    obj = {'content': b64}
    response = session.put("http://%s/%s/files/%s" % (ip, url_prefix, file_name), json=obj)
    _check_200(response)

def apply_config_from_internal_storage_persistently(file_name):
    # Send the request to apply the configuration file persistently.
    obj = {'file': file_name}
    response = session.post("http://%s/%s/configuration/startup" % (ip, url_prefix), json=obj)
    _check_200(response)

    id = response.json()['id']
    _poll_configuration_status(id)

def upload_and_apply_config_persistently(file_path):
    # Get the file name and content.
    dir_path, file_name = os.path.split(file_path)
    content = _load_file(file_path)
    b64 = _encode_base64(content)

    # Send the request including the config file.
    obj = {'name': file_name, 'content': b64}
    response = session.put("http://%s/%s/configuration/startup" % (ip, url_prefix), json=obj)
    _check_200(response)

    # Wait while busy..
    id = response.json()['id']
    _poll_configuration_status(id)

def get_software_versions():
    response = session.get("http://%s/%s/system/software" % (ip, url_prefix))
    _check_200(response)
    print(response.text)
    return response.json()

def install_software_update(file_path):
    if not os.path.exists(file_path):
        print("file '%s' does not exists" % file_path)
        exit()
    with open(file_path, "rb") as fh:
        try:
            data = fh.read()
        except Exception as e:
            print("file '%s' load failed '%s'" % (file_path, str(e)))
            exit()
    url = "http://%s/%s/system/software" % (ip, url_prefix)
    response = session.put(url=url, data=data)
    _check_200(response)

def reboot():
    response = session.post("http://%s/%s/system/reboot" % (ip, url_prefix))
    _check_200(response)




get_oapi_version()

login()

get_system_id()

#print_running_config()

#upload_file_to_internal_storage("/home/user/Downloads/test.cfg")
#apply_config_from_internal_storage_persistently("test.cfg")

#upload_and_apply_config_persistently("/home/user/Downloads/test.cfg")

#install_software_update("/home/user/Downloads/update.raucb")
#reboot()
